use std::fmt::Display;

#[derive(Debug, Clone)]
pub enum Symbol {
    Function(FunctionSymbol),
    Variable(VariableSymbol),
}

impl Symbol {
    pub fn name(&self) -> &str {
        match self {
            Symbol::Function(s) => s.declared_name.as_str(),
            Symbol::Variable(s) => s.name.as_str(),
        }
    }
}

impl Display for Symbol {
    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
        use Symbol::*;
        match self {
            Function(function_symbol) => write!(f, "{}", function_symbol),
            Variable(variable_symbol) => write!(f, "{}", variable_symbol),
        }
    }
}

#[derive(Debug, Clone)]
pub struct FunctionSymbol {
    pub fqn: String,
    pub declared_name: String,
    pub is_public: bool,
}

impl Display for FunctionSymbol {
    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
        write!(
            f,
            "FunctionSymbol(fqn = {}, declared_name = {}, is_public = {})",
            self.fqn, self.declared_name, self.is_public
        )
    }
}

#[derive(Debug, Clone)]
pub struct VariableSymbol {
    pub name: String,
    pub is_mutable: bool,
}

impl Display for VariableSymbol {
    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
        write!(
            f,
            "VariableSymbol(name = {}, is_mutable = {})",
            self.name, self.is_mutable
        )
    }
}